//
//  ViewController.swift
//  Angry Birds en la Tierra y la Luna
//
//  Created by Santiago Pavón Gómez on 29/9/16.
//  Copyright © 2016 UPM. All rights reserved.
//

import UIKit

class ViewController: UIViewController, TrajectoryViewDataSource {

    @IBOutlet weak var speedSlider: UISlider!
    @IBOutlet weak var angleSlider: UISlider!
    @IBOutlet weak var distanceSlider: UISlider!
    @IBOutlet weak var scaleSlider: UISlider!
    
    @IBOutlet weak var speedValue: UILabel!
    @IBOutlet weak var angleValue: UILabel!
    @IBOutlet weak var distanceValue: UILabel!
    @IBOutlet weak var scaleValue: UILabel!
    
    
    @IBOutlet weak var earthTrajView: TrajectoryView!
    @IBOutlet weak var moonTrajView: TrajectoryView!
    
    var earthTrajModel: TrajectoryModel!
    var moonTrajModel: TrajectoryModel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        earthTrajModel = TrajectoryModel(name: "La Tierra", gravity: 9.80665)
        moonTrajModel = TrajectoryModel(name: "La Luna", gravity: 1.622)
        
        earthTrajView.dataSource = self
        moonTrajView.dataSource = self
        
        speedSlider.sendActions(for: .valueChanged)
        angleSlider.sendActions(for: .valueChanged)
        distanceSlider.sendActions(for: .valueChanged)
        scaleSlider.sendActions(for: .valueChanged)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    
    // MARK: - Slider Actions
    
    @IBAction func speedChanged(_ sender: UISlider) {
        earthTrajModel.speed = Double(sender.value)
        moonTrajModel.speed = Double(sender.value)
        
        speedValue.text = String(format: "%.2f m/s", arguments: [sender.value])
        
        earthTrajView.setNeedsDisplay()
        moonTrajView.setNeedsDisplay()
    }
    
    @IBAction func angleChanged(_ sender: UISlider) {
        earthTrajModel.angle = Double(sender.value)
        moonTrajModel.angle = Double(sender.value)
        
        angleValue.text = String(format: "%.2f rad", arguments: [sender.value])
        
        earthTrajView.setNeedsDisplay()
        moonTrajView.setNeedsDisplay()
    }
    
    @IBAction func distanceChanged(_ sender: UISlider) {
        distanceValue.text = String(format: "%.2f m", arguments: [sender.value])
        
        earthTrajView.distanceToTarget = Double(sender.value)
        moonTrajView.distanceToTarget = Double(sender.value)
    }
    
    @IBAction func scaleChanged(_ sender: UISlider) {
        let scale = Double(sender.value)
        scaleValue.text = String(format: "%.2f mpp", arguments: [scale])
        earthTrajView.scale = scale
        moonTrajView.scale = scale
    }
    
    // MARK: - Reconocedores de gestos

    /// Con un gesto pan actualizo la verlocidad y el ángulo.
    @IBAction func processPan(_ sender: UIPanGestureRecognizer) {
        
        guard let view = sender.view else { return }
        
        // Calcular los valores a poner en las sliders.
        var newSpeed = speedSlider.value + Float(sender.translation(in: view).x)
        newSpeed = min(max(newSpeed, 10), 500)
        
        var newAngle = angleSlider.value - Float(sender.translation(in: view).y / view.bounds.size.height) * .pi / 2 / 2
        newAngle = min(max(newAngle, 0.1), .pi / 2)
        
        sender.setTranslation(CGPoint.zero, in: view)
        
        // Poner los valores nuevos en las sliders.
        speedSlider.value = newSpeed
        angleSlider.value = newAngle

        // Ejecutar las accciones registradas en las sliders.
        speedSlider.sendActions(for: .valueChanged)
        angleSlider.sendActions(for: .valueChanged)
    }
    
    /// Con un gesto pinch cambio el zoom o la escala.
    @IBAction func processPinch(_ sender: UIPinchGestureRecognizer) {
        
        // Calcular el valor a poner en la slider.
        let newScale = scaleSlider.value / Float(sender.scale)
        sender.scale = 1
        
        // Poner el valor nuevo en la slider.
        scaleSlider.value = newScale
        
        // Ejecutar las accciones registradas en la slider.
        scaleSlider.sendActions(for: .valueChanged)
    }
    
    /// Con un gesto rotation cambio el ángulo inicial de disparo.
    @IBAction func processRotation(_ sender: UIRotationGestureRecognizer) {
        
        // Calcular el valor a poner en la slider.
        var newAngle = angleSlider.value + Float(sender.rotation)
        newAngle = min(max(newAngle, 0.1), Float.pi / 2)
        sender.rotation = 0
        
        // Poner el valor nuevo en la slider.
        angleSlider.value = newAngle

        // Ejecutar las accciones registradas en la slider.
        angleSlider.sendActions(for: .valueChanged)
    }
    
    /// Con un gesto LongPress sitio el objetivo.
    @IBAction func processLongPress(_ sender: UILongPressGestureRecognizer) {
        
        guard let view = sender.view else { return }

        // Calcular el valor a poner en la slider.
        let x = Float(sender.location(in: view).x) - Float(LEFT_MARGIN)
        let newDistance = max(x * scaleSlider.value, 1)
        
        // Poner el valor nuevo en la slider.
        distanceSlider.value = newDistance
        
        // Ejecutar las accciones registradas en la slider.
        distanceSlider.sendActions(for: .valueChanged)
    }
    
    
    
    
    
    // MARK: - TrajectoryViewDataSource {
    
    func startTime(for trajectoryView: TrajectoryView) -> Double {
        return 0
    }
    
    func endTime(for trajectoryView: TrajectoryView) -> Double {
        
        switch trajectoryView {
        case earthTrajView:
            return earthTrajModel.timeToTarget()
        case moonTrajView:
            return moonTrajModel.timeToTarget()
        default:
            return 0.0
        }
    }
    
    func position(for trajectoryView: TrajectoryView, at time: Double) -> Position {
        
        switch trajectoryView {
        case earthTrajView:
            let distance = earthTrajModel.distanceAtTime(time: time)
            let alture = earthTrajModel.altureAtTime(time: time)
            return Position(distance: distance, alture: alture)
        case moonTrajView:
            let distance = moonTrajModel.distanceAtTime(time: time)
            let alture = moonTrajModel.altureAtTime(time: time)
            return Position(distance: distance, alture: alture)
        default:
            return Position(distance: 0, alture: 0)
        }
    }

}

